% bartlett_generation:
%
% Generate the Bartlett parameters (coefficients 'b' and error variances
% 'gamma' for each valid instantiation of z (as given by 'maximal_z'),
% with respect to a given variable Y_i.%
%
% Author: Ricardo Silva (ricardo@stats.ucl.ac.uk)
% Last modification: Oct 07 2008
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Inputs:
% 
% - i: target variable


function [b, gamma] = bartlett_generation(i, subset)

global vars;

n = length(subset);

if n == 1
    b = [];
    gamma = vars{i};
    return;
end

[maximal_z z_universe] = get_z_range(subset);

num_spouses = n - 1;
b = zeros(num_spouses, maximal_z);
gamma = zeros(maximal_z, 1);

i_pos = find(subset == i);
other_pos = setdiff(1:n, i_pos);

for zi = z_universe
    current_z = get_z(subset, zi);
    impl_cov = assemble_cov(subset, current_z);    
    cov_i_others = impl_cov(i_pos, other_pos);
    cov_others = impl_cov(other_pos, other_pos);
    inv_cov_others = inv(cov_others);
    b(:, zi) = cov_i_others * inv_cov_others;
    gamma(zi) = impl_cov(i_pos, i_pos) - cov_i_others * b(:, zi);
end
