% get_quad_var_constr:
%
% Evaluate all the variance constraints, which are quadratic on the Bartlett
% coefficients.

function [c, ceq, cg, ceqg] = get_quad_var_constraints(params)

global target_subset;
global fixed;
global vars;
global pi_z;
global target_var;

subset = target_subset;
maximal_z = get_z_range(subset);

% No inequalities
c = []; cg = [];

% Initialization
n = length(subset);
other_subset = setdiff(subset, target_var);
pos_target = find(subset == target_var);
other_current_z = ones(n - 1, 1);
other_pos = zeros(length(other_subset), 1);
for o = 1:length(other_subset)
    other_pos(o) = find(subset == other_subset(o));
end

[other_maximal_z other_z_universe] = get_z_range(other_subset);
other_z_universe = setdiff(other_z_universe, 1);

% Find number of constraints, allocate memory
num_constraints = length(pi_z{target_var}) * (other_maximal_z - 1);
if fixed(pos_target, pos_target)
   num_constraints = num_constraints + length(pi_z{target_var});
end
ceq = zeros(num_constraints, 1);
ceqg = zeros(n * maximal_z, num_constraints);
num_constraints = 0; % Reset it

% Equalities, part 1: if target variable has its variances frozen.
if fixed(pos_target, pos_target)
    for z = 1:length(pi_z{target_var})
        num_constraints = num_constraints + 1;
        
        %Find value of the constraint        
        current_z = ones(n, 1); 
        current_z(pos_target) = z;
        impl_cov = assemble_cov(other_subset, other_current_z); 
        start = get_z_inverse(subset, current_z);
        b       = params(((start - 1) * n + 1):(start * n - 1));
        gamma   = params(start * n);
        implied = b * impl_cov * b' + gamma;
        ceq(num_constraints) = implied - vars{target_var}(z);
        
        %Find gradient of the constraint
        gr = zeros(1, n * maximal_z);
        gr(((start - 1) * n + 1):(start * n - 1)) = 2 * b * impl_cov;
        gr(start * n) = 1.;
        ceqg(:, num_constraints) = gr;
    end   
end

% Equalities, part 2: variances are sensitive only to changes in the
% mixture indicator of the target variable
i_pos = find(subset == target_var);

for z = 1:length(pi_z{target_var})

    z_refvalues = ones(n, 1);
    z_refvalues(i_pos) = z;

    start_ref    = get_z_inverse(subset, z_refvalues);
    impl_cov_ref = assemble_cov(other_subset, z_refvalues(other_pos)); 
    b_ref        = params(((start_ref - 1) * n + 1):(start_ref * n - 1));
    gamma_ref    = params(start_ref * n);

    % Now, iterate through all possible values of the mixture
    % indicators of the other variables (except the reference
    % class)

    for zi_other = other_z_universe
        num_constraints = num_constraints + 1;
        
        %Find the corresponding z_index for that
        z_value_other       = get_z(other_subset, zi_other);
        z_values            = zeros(n, 1);
        z_values(i_pos)     = z;
        z_values(other_pos) = z_value_other;

        % Now, calculate this constraint        
        start_other    = get_z_inverse(subset, z_values);
        impl_cov_other = assemble_cov(other_subset, z_values(other_pos)); 
        b_other        = params(((start_other - 1) * n + 1):(start_other * n - 1));
        gamma_other    = params(start_other * n);
        
        v = (b_ref   * impl_cov_ref   * b_ref'   + gamma_ref) - ...
            (b_other * impl_cov_other * b_other' + gamma_other);
        ceq(num_constraints) = v;
        
        %Find gradient of the constraint
        gr = zeros(1, n * maximal_z);
        gr(((start_ref - 1) * n + 1):(start_ref * n - 1))     = 2 * b_ref * impl_cov_ref;
        gr(((start_other - 1) * n + 1):(start_other * n - 1)) = -2 * b_other * impl_cov_other;
        gr(start_ref * n) = 1.;
        gr(start_other * n) = -1.;
        ceqg(:, num_constraints) = gr;
    end
end
