% get_z:
%
% Convert the numerical value encoding a combination of mixture indicators
% into the correspondent vector of indicators.
%
% Author: Ricardo Silva (ricardo@stats.ucl.ac.uk)
% Last modification: Oct 07 2008
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Inputs:
%
% - value: a number between 1 and the total number of combinations.
%          The total number of combinations is encoded in the length of
%          the cell elements of pi_z. We assume 'value' starts from 1.

function [output] = get_z(subset, value)

global pi_z;

n = length(subset);
output = zeros(n, 1);

for i = n:-1:1
    num_components_left = 1;
    for j = 1:(i - 1)
        num_components_left = num_components_left * length(pi_z{subset(j)});
    end
    q = ceil(value / num_components_left);
    output(i) = q;
    value = value - (q - 1) * num_components_left;
end


