% Learn the i-th row of all possible covariance matrices within region
% defined by 'subset'.
%
% Several of the entries are fixed, which will imply linear and
% quadratic constraints on the Bartlett parameters.
%
% IMPORTANT: it is assumed that 'b' is a K x S matrix where 

% In this code, variances and covariances are fit together. It can get
% REALLY slow, particularly because I'm yet to implement the code to
% calculate gradients analytically.

function [new_b, new_gamma] = learn_covariances(i, subset, fixed_in, b, gamma)
               
global target_var; global target_subset;
global fixed;

fixed = fixed_in;
target_var = i;
target_subset = subset;

[maximal_z z_universe] = get_z_range(subset);

% Common base case: region has only one element
n = length(subset);
if n == 1
   if fixed(1, 1) % i.e., region is a copy of a child, hence no learning
       return;
   end
   new_b = [];
   new_gamma = learn_single_variance(i);
   return
end

% Get matrix of linear equalities due to parameter fixing
[Aeq1 beq1] = get_fixing_cov_constraints(i, subset, fixed);
         
% Get matrix of linear equalities due to mixture indicators
[Aeq2 beq2] = get_mixture_cov_constraints(i, subset);
% The nasty quadratic constraints are defined by the function
% 'get_quad_var_constraints' instead.

% Get initial parameter vectors, boundary and and linear constraints

initial_params = zeros(1, maximal_z * n);
A = zeros(maximal_z, maximal_z * n);
Ab = -0.0001 * ones(maximal_z,1);
for zi = z_universe
    st = (zi - 1) * n + 1;
    ed = zi * n;
    initial_params(st:(ed - 1)) = b(:, zi);
    initial_params(ed) = gamma(zi);
    A(zi, ed) = -1;
end

Aeq = [Aeq1; Aeq2];
beq = [beq1; beq2];

% Call the optimization routine

options = optimset('GradObj','on', 'LargeScale', 'off', ...
                   'GradConstr', 'on', 'DerivativeCheck', 'off', ...
                   'Display', 'iter', 'MaxFunEvals', 5000);

[out_params, fval, exitflag] = fmincon(@conditional_fitting_function,  ...
                               initial_params, A, Ab, Aeq, beq, [], [], ...
                               @get_quad_var_constraints, options);
if exitflag < 0
   warning('learn_covariances:var', 'Optimization did not converge!');
end

% Rebuild parameters from optimization output

new_b = zeros(n - 1, maximal_z);
new_gamma = zeros(maximal_z, 1);
for zi = z_universe
    st = (zi - 1) * n + 1;
    ed = zi * n;
    new_b(:, zi) = out_params(st:(ed - 1));
    new_gamma(zi) = out_params(ed);
end

rebuild_from_bartlett(i, subset, new_b, new_gamma);

clear global target_var; clear global target_subset;
clear global fixed;
