% em_score:
%
% EM score: assumes as input the posterior distribution of z's given
% previous parameter instantiation.
%
% Author: Ricardo Silva (ricardo@stats.ucl.ac.uk)
% Last modification: Oct 07 2008
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [score] = local_em_score(subset)
                        
global data;    
global pi_z;
global z_post;

[maximal_z z_universe] = get_z_range(subset);

score = 0;
sample_size = size(data, 1);
n = length(subset);

% Likelihood term

for zi = z_universe
    current_z = get_z(subset, zi);    
    if prob_zero(subset, current_z) == 0
       continue;
    end;
    impl_cov = assemble_cov(subset, current_z);
    impl_mean_t = assemble_mean(subset, current_z);
    inv_impl = inv(impl_cov);
    logdet = log(det(impl_cov));
    
    sum_p = 0.;
    mean_buffer_m = zeros(n, n);
    for d = 1:sample_size
        p_z = z_post(d, zi);
        sum_p = sum_p + p_z;
        mean_buffer = data(d, subset)' - impl_mean_t;
        mean_buffer_m = mean_buffer_m + p_z * mean_buffer * mean_buffer';
        for c = 1:n
            score = score + log(pi_z{subset(c)}(current_z(c))) * p_z;
            if (isnan(score)) 
                error('EMScore == NaN');
            end
        end
    end
    
    score = score - 0.5 * (logdet * sum_p + trace(mean_buffer_m * inv_impl));
    if (isnan(score)) 
        error('EMScore == NaN');
    end
end

score = score / sample_size;

