% lvm_create_params:
%
% Allocate and initialize parameter cells. This is the latent mixture of
% Gaussians models with one latent per clique.

function [intercepts, coeffs, gammas, pi] = lvm_create_params(data, graph)

global num_components;
num_vars = length(graph);

[cliques, intersections, clique_parents] = build_clique_information(graph);
intercepts = cell(num_vars, 1);
gammas = cell(num_vars, 1);
coeffs = cell(num_vars, 1);
for i = 1:num_vars
    intercepts{i} = zeros(num_components(i), 1);
    gammas{i} = zeros(num_components(i), 1);
end

for i = 1:num_vars
    coeffs{i} = rand(length(clique_parents{i}), num_components(i)) / length(clique_parents{i});
end

for i = 1:num_vars
    y_c = kMeansCluster(data(:, i), num_components);
    total_v = num_components(num_vars);
    for v = 1:total_v
        gammas{i}(v) = 1;
        intercepts{i}(v) = mean(y_c(y_c(:, 2) == v, 1));
    end
end

pi = cell(num_vars, 1);
for i = 1:num_vars
    pi{i} = ones(num_components(i), 1);
    pi{i} = pi{i} / sum(pi{i});
end
