% The 'expectation' step of the EM algorithm: cache all posterior
% distributions for all possible z instantiations and data points.
%
% Needless to say, this is very memory and cpu intense.

function [cached_z] = lvm_expected_z()

global data;    
global pi_z;
global maximal_z;
global z_universe;

sample_size = size(data, 1);
num_vars = size(data, 2);
cached_z = zeros(sample_size, maximal_z);

for zi = z_universe
    current_z = get_z(1:num_vars, zi);
    log_prob_zi = 0;
    for i = 1:num_vars
        log_prob_zi = log_prob_zi + log(pi_z{i}(current_z(i)));
    end
    impl_cov  = lvm_assemble_cov(current_z);
    impl_mean = lvm_assemble_mean(current_z);
    inv_impl  = inv(impl_cov);
    logdet    = log(det(impl_cov));
    if (isnan(logdet))
        error('NaN here!');
    end
    for d = 1:sample_size
        cached_z(d, zi) = fast_llik(data(d, :)', logdet, inv_impl, impl_mean) + log_prob_zi; 
    end
end

for d = 1:sample_size
    cached_z(d, :) = cached_z(d, :) - max(cached_z(d, :));
    cached_z(d, :) = exp(cached_z(d, :));
    cached_z(d, :) = cached_z(d, :) / sum(cached_z(d, :));
end
