% neg_gamma_llik_fast:
%
% Negative conditional log-likelihood for all gammas given some external
% sufficient statistics. Used when learning gamma parameters.
%
% Author: Ricardo Silva (ricardo@stats.ucl.ac.uk)
% Last modification: Oct 08 2008
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Inputs:
%
% - gamma: current value of the parameters;

function [f g] = neg_gamma_llik_fast(gamma)

global offset_matrix; global gamma_reference;

if sum(gamma <= eps) > 0 
    f = Inf;
    if nargout > 1
       g = zeros(length(gamma), 1);
    end
    return;
end

global ln_gamma_coeff;
global inv_gamma_coeff;
global data;

sample_size = size(data, 1);
new_gamma = gamma_reference * gamma + offset_matrix;

%Likelihood
f = log(new_gamma)' * ln_gamma_coeff + (1 ./ new_gamma)' * inv_gamma_coeff;

f = f / sample_size;

if isnan(f)
    f = Inf;
    if nargout > 1
       g = zeros(length(gamma), 1);
    end
    return;
end

if nargout > 1
    num_gamma = length(gamma);
    g = zeros(num_gamma, 1);

    %Likelihood
    for i = 1:num_gamma
        terms = find(gamma_reference(:, i) == 1);
        %Likelihood
        g(i) = (1 ./ new_gamma(terms))' * ln_gamma_coeff(terms) - ...
               (1 ./ new_gamma(terms).^2)' * inv_gamma_coeff(terms);
    end
    
    g = g ./ sample_size;
end
