% predictive_llik:
% predictive log-likelihood on testset 'test'

function result = predictive_llik(test, meansI, varsI, covsI, pi_zI, ...
                                  graphI, num_componentsI)

disp('Calculating predictive log-likelihood (bi-directed model)...');

global means;   global vars;          global covs;  global pi_z;
means = meansI; vars = varsI; covs = covsI; pi_z = pi_zI;

global graph;   global data; global num_components;
graph = graphI; data = test; num_components = num_componentsI;

% Initialization
num_vars = length(graph);
test_size = size(test, 1);
[maximal_z z_universe] = get_z_range(1:num_vars);
result = 0;

for d = 1:test_size
    pred = zeros(maximal_z, 1);
    prob = zeros(maximal_z, 1);
    for zi = z_universe
        current_z = get_z(1:num_vars, zi);

        prob(zi) = 1;
        for c = 1:num_vars
            prob(zi) = prob(zi) * pi_z{c}(current_z(c));
        end    
        if prob(zi) == 0
           pred(zi) = -Inf;
           continue
        end
        
        impl_mean = assemble_mean(1:num_vars, current_z);
        impl_cov  = assemble_cov(1:num_vars, current_z);
        impl_det  = det(impl_cov);
        logdet = log(impl_det);
        inv_impl = inv(impl_cov);    
        dmean = test(d, :)' - impl_mean;
        
        pred(zi) = log(prob(zi)) -0.5 * (num_vars * log(2 * pi) + ...
                   logdet + trace(inv_impl *(dmean * dmean')));
    end    
    
    pred = pred(prob > 0);
    max_pred = max(pred);
    pred = pred - max_pred; 
    result = result + log(sum(exp(pred))) + max_pred;
end

result = result / test_size;
