% get_initial_gamma:
%
% Start from a gamma that might not be consistent, then project it
% to a gamma vector that respects the equality constraints.
%
% Author: Ricardo Silva (ricardo@stats.ucl.ac.uk)
% Last modification: Oct 07 2008
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Inputs:
%
% - i: target variable

function output_gamma = get_initial_gamma(i, subset, gamma, b)

global graph;
global pi_z;
global covs;

[maximal_z z_universe] = get_z_range(subset);

output_gamma = zeros(maximal_z, 1);
other_subset = setdiff(subset, i);

i_pos = find(subset == i);

%For each assignment of the parent cliques
num_v = length(pi_z{i});
for v = 1:num_v
    all_offsets = -Inf * ones(maximal_z, 1);
    for zi = z_universe
       z_inst = get_z(subset, zi);
       if z_inst(i_pos) ~= v
          continue;
       end
       all_offsets(zi) = 0;
       for j = other_subset
           if graph(i, j) == 0
              continue
           end
           j_other_pos = find(other_subset == j);
           j_pos = find(subset == j);
           pi = get_parameter_index(i, j, z_inst(i_pos), z_inst(j_pos));
           all_offsets(zi) = all_offsets(zi) + b(j_other_pos, zi) * covs{i, j}(pi);
       end
    end
    [reference_offset z_refp] = max(all_offsets); %Choose equivalence class    
    output_gamma(z_refp) = gamma(z_refp);
    
    %Iterate over all z configurations with the same v assignment
    for zi = z_universe
        if zi == z_refp
           continue;
        end
        z_inst = get_z(subset, zi);
        if z_inst(i_pos) ~= v
           continue;
        end
        %agreement found        
        output_gamma(zi) = output_gamma(z_refp) + reference_offset - all_offsets(zi);
    end       
end
